"use strict";
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseClient = void 0;
const Constants_1 = require("../utils/Constants");
const axios_1 = __importDefault(require("axios"));
const https_1 = require("https");
const eventemitter3_1 = require("eventemitter3");
const url_1 = require("url");
const Methods_1 = require("../utils/Methods");
const ErrorHandler_1 = require("../errors/ErrorHandler");
/**
 * Base Client for the Client to extend
 * @extends EventEmitter
 */
class BaseClient extends eventemitter3_1.EventEmitter {
    constructor(options = {}, payload = {}) {
        var _a, _b;
        super();
        this._bearer = null;
        // Set defaults
        this._basic = options.basic || (0, Methods_1.createBasicAuth)();
        this._bearer = options.bearer || null;
        const headers = (_a = options === null || options === void 0 ? void 0 : options.headers) !== null && _a !== void 0 ? _a : Constants_1.DefaultHeaders;
        headers["user-agent"] = (_b = options === null || options === void 0 ? void 0 : options.userAgent) !== null && _b !== void 0 ? _b : Constants_1.DefaultUserAgent;
        this.instance = axios_1.default.create({
            baseURL: `https://${Constants_1.APIBase}/v${Constants_1.APIVersion}`,
            headers: headers,
            httpsAgent: new https_1.Agent({ keepAlive: true }),
        });
        this.instance.interceptors.request.use((config) => {
            var _a;
            (_a = config.headers) !== null && _a !== void 0 ? _a : (config.headers = headers);
            config.headers.Authorization = this.authorization;
            //console.log(`set headers:\n ${JSON.stringify(config.headers, null, 2)}`);
            return config;
        });
        this.instance.interceptors.response.use((response) => response, (error) => {
            throw (0, ErrorHandler_1.handleError)(error);
        });
        this._payload = Object.assign({}, payload);
    }
    /**
     * The authorization string used for requests
     */
    get authorization() {
        return this._bearer ? `Bearer ${this._bearer}` : `Basic ${this._basic}`;
    }
    /**
     * Gets the payload
     */
    get payload() {
        return this._payload;
    }
    set payload(value) {
        this._payload = Object.assign(this.payload, value);
    }
    /**
     * Calls eval with the client set as the "this" context.
     * @param script The script to evaluate
     * @returns The result of the script
     */
    _eval(script) {
        return eval(script);
    }
    /**
     * Paginates through items in a REST API endpoint.
     * @param url The url to get
     * @param key The key to get the array of values from
     * @param params The params to pass to the url
     * @yields The value of the key
     */
    paginate(url, key, params) {
        var _a, _b, _c, _d, _e, _f;
        return __asyncGenerator(this, arguments, function* paginate_1() {
            if (!url)
                throw new Error("paginate: url is required");
            if (!key)
                throw new Error("paginate: key is required");
            let method = "GET";
            if (url.toLowerCase() === "collective") {
                method = "POST";
            }
            let hasNext;
            if (!params)
                params = new url_1.URLSearchParams();
            let config = method === "GET"
                ? { method, url, params: new url_1.URLSearchParams(params) }
                : { method, url, data: params || {} };
            do {
                let { data } = yield __await(this.instance.request(config));
                const items = (_b = (_a = data === null || data === void 0 ? void 0 : data.data) === null || _a === void 0 ? void 0 : _a[key]) === null || _b === void 0 ? void 0 : _b.items;
                if (!Array.isArray(items))
                    throw new Error("paginate: items is not an array");
                hasNext = (_f = (_e = (_d = (_c = data === null || data === void 0 ? void 0 : data.data) === null || _c === void 0 ? void 0 : _c[key]) === null || _d === void 0 ? void 0 : _d.paging) === null || _e === void 0 ? void 0 : _e.has_next) !== null && _f !== void 0 ? _f : false;
                if (hasNext) {
                    params.set("next", data.data[key].paging.cursors.next);
                }
                for (let item of items) {
                    yield yield __await(item);
                }
            } while (hasNext);
        });
    }
}
exports.BaseClient = BaseClient;
//# sourceMappingURL=BaseClient.js.map