"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBasicAuth = exports.sleep = exports.dateFromTimestamp = exports.isAxiosError = exports.assertError = exports.assert = exports.ifModifiedSince = void 0;
const axios_1 = __importDefault(require("axios"));
const Constants_1 = require("./Constants");
const crypto_1 = __importDefault(require("crypto"));
const iFunnyError_1 = require("../errors/iFunnyError");
const iFunnyErrorCodes_1 = require("../errors/iFunnyErrorCodes");
/**
 * Certain requests have a header for 'if-modified-since' which uses the current timestamp
 * @param date Custom date
 * @returns GMT timestamp of the date
 */
function ifModifiedSince(date) {
    date !== null && date !== void 0 ? date : (date = new Date());
    return date.toUTCString();
}
exports.ifModifiedSince = ifModifiedSince;
/**
 * Asserts a claim to be true
 * @param claim The claim to check
 * @param error The message to send if the claim evaluted to false
 */
function assert(claim, error) {
    if (!claim) {
        if (typeof error === "string") {
            throw new Error(error);
        }
        else if (error instanceof Error) {
            throw error;
        }
    }
    return true;
}
exports.assert = assert;
/**
 *
 * @param item The item to check
 * @returns
 */
function assertError(item) {
    return assert(item instanceof Error, `Unknown Error: ${item}`);
}
exports.assertError = assertError;
function isAxiosError(err) {
    return axios_1.default.isAxiosError(err);
}
exports.isAxiosError = isAxiosError;
/**
 * Returns a Date object from a timestamp
 * @param timestamp Number of seconds of the timestamp
 * @param inMilliseconds Whether the timestamp is already in milliseconds or not
 * @returns
 */
function dateFromTimestamp(timestamp, inMilliseconds = true) {
    // ? Converts timestamp to milliseconds if necessary
    if (!inMilliseconds)
        timestamp *= 1000;
    return new Date(timestamp);
}
exports.dateFromTimestamp = dateFromTimestamp;
/**
 * Sleeps the program for the specified amount of milliseconds
 * @param ms The number of milliseconds to wait
 * @returns A promise that resolves after the given amount of time
 */
function sleep(ms) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve) => setTimeout(resolve, ms));
    });
}
exports.sleep = sleep;
/**
 * Generates a basic token
 * @param opts Options to create the token with
 * @param opts.clientId The client id to use for the token
 * @param opts.clientSecret The client secret to use for the token
 * @param opts.length The length of the token to create
 * @returns
 */
function createBasicAuth(opts) {
    const { length, clientId, clientSecret } = Object.assign(Constants_1.DefaultBasicAuthConfig, opts);
    let uuid = crypto_1.default.randomUUID().replace(/\-/g, "");
    let hex;
    switch (length) {
        case 112:
            hex = uuid.toUpperCase();
            break;
        case 156:
            hex = crypto_1.default.createHash("sha256").update(uuid).digest("hex").toUpperCase();
            break;
        default:
            throw new iFunnyError_1.iFunnyError(iFunnyErrorCodes_1.iFunnyErrorCodes.InvalidBasicTokenLength);
    }
    let a = hex + `_${clientId}:`;
    let b = hex + `:${clientId}:${clientSecret}`;
    let c = crypto_1.default.createHash("sha1").update(b).digest("hex");
    return Buffer.from(a + c).toString("base64");
}
exports.createBasicAuth = createBasicAuth;
//# sourceMappingURL=Methods.js.map